@@ -11,6 +11,7 @@ use rust_animation::play::Play;
 use rust_animation::stage::Stage;
 use rust_animation::actor::Actor;
 use rust_animation::actor::EasingFunction;
+use rust_animation::actor::LayoutMode;
 
 fn main() {
   let mut glfw = glfw::init(glfw::FAIL_ON_ERRORS).unwrap();
@@ -31,7 +32,7 @@ fn main() {
 
   let mut play = Play::new("Animation test".to_string());
   play.initialize();
-  let mut stage = Stage::new("stage".to_string(), 1920, 1080, None);
+  let mut stage = Stage::new("stage".to_string(), 1920, 1080, LayoutMode::UserDefine, None);
   stage.set_visible(true);
 
   let mut actor = Actor::new("actor_1".to_string(), 400, 225, None);
@@ -55,6 +56,13 @@ fn main() {
   // 0 degree -> 360 degree for 5 sec
   actor_2.apply_rotation_animation(0, 360, 5.0, EasingFunction::EaseInOut);
 
+
+  let mut actor_3 = Play::new_actor("actor_3".to_string(), 50, 50, None);
+  actor_3.x = 10;
+  actor_3.y = 10;
+  actor_3.set_color(1.0, 0.0, 0.0);
+  actor_2.add_sub_actor(actor_3);
+
   stage.add_actor(actor);
   stage.add_actor(actor_2);
   play.add_stage(stage);
