@@ -16,6 +16,7 @@ use std::ptr;
 use std::time::{Instant};
 
 use keyframe::{ease, functions::*};
+use stretch::{style::*, node::{Node, Stretch}, geometry::Size, geometry::Rect};
 
 use crate::actor::image::GenericImage;
 
@@ -40,6 +41,12 @@ pub enum EasingFunction {
     Step,
 }
 
+#[derive(Copy, Clone, Debug)]
+pub enum LayoutMode {
+  UserDefine,
+  Flex
+}
+
 macro_rules! c_str {
   ($literal:expr) => {
       CStr::from_bytes_with_nul_unchecked(concat!($literal, "\0").as_bytes())
@@ -100,7 +107,9 @@ pub struct Actor<'a> {
   layout: Option<Box<dyn Layout + 'a>>,
   focused_sub_actor: usize,
   focused: bool,
-  needsUpdate: bool
+  needsUpdate: bool,
+  pub node: Option<Node>,
+  style: Option<Style>
 }
 
 pub trait EventHandler {
@@ -163,7 +172,9 @@ impl<'a> Actor<'a> {
       layout: None,
       focused_sub_actor: 0,
       focused: false,
-      needsUpdate: false
+      needsUpdate: false,
+      node: None,
+      style: None
     }
   }
 
@@ -171,6 +182,8 @@ impl<'a> Actor<'a> {
     self.viewport_width = viewport_width;
     self.viewport_height = viewport_height;
 
+    println!("init_gl {}", self.name);
+
     unsafe {
       let (mut vertex_array_buffer, mut elem_array_buffer) = (0, 0);
       let vertices: [f32; 20] = [
@@ -259,33 +272,35 @@ impl<'a> Actor<'a> {
     self.layout = layout;
   }
 
+  pub fn set_style(&mut self, style: Style) {
+    self.style = Some(style);
+  }
+
   /*pub fn update(&mut self) {
     // Sort sub actors by z-axis
     self.sub_actor_list.sort_by(|a, b| a.z.partial_cmp(&b.z).unwrap());
   }*/
 
   fn easing_function(easing: EasingFunction, from: f32, to: f32, duration: f32) -> f32 {
-     let val : f32;
      match easing {
-        EasingFunction::EaseIn => { val = ease(EaseIn, from, to, duration); }
-        EasingFunction::EaseInCubic  => { val = ease(EaseInCubic, from, to, duration); }
-        EasingFunction::EaseInOut => { val = ease(EaseInOut, from, to, duration); }
-        EasingFunction::EaseInOutCubic => { val = ease(EaseInOutCubic, from, to, duration); }
-        EasingFunction::EaseInOutQuad => { val = ease(EaseInOutQuad , from, to, duration); }
-        EasingFunction::EaseInOutQuart => { val = ease(EaseInOutQuart, from, to, duration); }
-        EasingFunction::EaseInOutQuint => { val = ease(EaseInOutQuint, from, to, duration); }
-        EasingFunction::EaseInQuad => { val = ease(EaseInQuad, from, to, duration); }
-        EasingFunction::EaseInQuart => { val = ease(EaseInQuart, from, to, duration); }
-        EasingFunction::EaseInQuint => { val = ease(EaseInQuint, from, to, duration); }
-        EasingFunction::EaseOut => { val = ease(EaseOut, from, to, duration); }
-        EasingFunction::EaseOutCubic => { val = ease(EaseOutCubic, from, to, duration); }
-        EasingFunction::EaseOutQuad => { val = ease(EaseOutQuad, from, to, duration); }
-        EasingFunction::EaseOutQuart => { val = ease(EaseOutQuart, from, to, duration); }
-        EasingFunction::EaseOutQuint => { val = ease(EaseOutQuint, from, to, duration); }
-        EasingFunction::Linear =>{ val = ease(Linear, from, to, duration); }
-        EasingFunction::Step => { val = ease(Step, from, to, duration); }
+        EasingFunction::EaseIn => ease(EaseIn, from, to, duration),
+        EasingFunction::EaseInCubic  => ease(EaseInCubic, from, to, duration),
+        EasingFunction::EaseInOut => ease(EaseInOut, from, to, duration),
+        EasingFunction::EaseInOutCubic => ease(EaseInOutCubic, from, to, duration),
+        EasingFunction::EaseInOutQuad => ease(EaseInOutQuad , from, to, duration),
+        EasingFunction::EaseInOutQuart => ease(EaseInOutQuart, from, to, duration),
+        EasingFunction::EaseInOutQuint => ease(EaseInOutQuint, from, to, duration),
+        EasingFunction::EaseInQuad => ease(EaseInQuad, from, to, duration),
+        EasingFunction::EaseInQuart => ease(EaseInQuart, from, to, duration),
+        EasingFunction::EaseInQuint => ease(EaseInQuint, from, to, duration),
+        EasingFunction::EaseOut => ease(EaseOut, from, to, duration),
+        EasingFunction::EaseOutCubic => ease(EaseOutCubic, from, to, duration),
+        EasingFunction::EaseOutQuad => ease(EaseOutQuad, from, to, duration),
+        EasingFunction::EaseOutQuart => ease(EaseOutQuart, from, to, duration),
+        EasingFunction::EaseOutQuint => ease(EaseOutQuint, from, to, duration),
+        EasingFunction::Linear => ease(Linear, from, to, duration),
+        EasingFunction::Step => ease(Step, from, to, duration)
      }
-     val
   }
   pub fn animate(&mut self) {
     if self.needsUpdate {
@@ -451,8 +466,40 @@ impl<'a> Actor<'a> {
   }
 
   // Marks the layer’s contents as needing to be updated.
-  pub fn set_needs_layout(&mut self) {
+  pub fn set_needs_layout(&mut self, stretch: &mut Option<Stretch>) {
      self.needsUpdate = true;
+
+    if let Some(stretch_obj) = stretch {
+      if let Some(style_obj) = self.style {
+        self.node = Some(stretch_obj.new_node(style_obj, vec![]).unwrap());
+      } else {
+        println!("default style: {}: {},{}", self.name, self.width, self.height);
+        self.node = Some(stretch_obj.new_node(Style {
+          size: Size { 
+              width: Dimension::Points(self.width as f32), 
+              height: Dimension::Points(self.height as f32),
+          },
+          margin: Rect {
+              start: Dimension::Points(2.0),
+              end: Dimension::Points(2.0),
+              top: Dimension::Points(2.0),
+              bottom: Dimension::Points(2.0),
+              ..Default::default()
+          },
+          ..Default::default()
+        }, vec![]).unwrap());
+      }
+    }
+    for sub_actor in self.sub_actor_list.iter_mut() {
+       sub_actor.set_needs_layout(stretch);
+      if !self.node.is_none() {
+        match stretch.as_mut().unwrap().add_child(self.node.unwrap(),
+          sub_actor.node.unwrap()) {
+          Ok(()) => { println!(" stretch node  is added {} {}", self.name, sub_actor.name)}
+          Err(..) => {}
+        }
+      }
+    }
   }
 
   // layout sub-actors.
@@ -462,15 +509,31 @@ impl<'a> Actor<'a> {
     }
   }
 
-  pub fn render(&self, shader_program: GLuint, actor: Option<&Actor>) {
-    let mut x : f32 = self.x as f32;
-    let mut y : f32 = self.y as f32;
+  pub fn update_layout(&mut self, stretch: &mut Option<Stretch>) {
+    // If Stretch's node is set, Stretch does a layout job.
+    if let Some(stretch_obj) = stretch {
+      let layout = stretch_obj.layout(self.node.unwrap()).unwrap();
+      self.x = layout.location.x as i32;
+      self.y = layout.location.y as i32;
+      //println!("node: {:#?}", stretch_obj.layout(self.node.unwrap())); 
+  
+      for sub_actor in self.sub_actor_list.iter_mut() {
+        sub_actor.update_layout(stretch);
+      }
+    }
+  }
+
+  pub fn render(&self, shader_program: GLuint, stretch: &mut Option<Stretch>, actor: Option<&Actor>) {
+    let mut x = self.x;
+    let mut y = self.y;
 
     if let Some(main_actor) = actor {
-      x += main_actor.x as f32;
-      y += main_actor.y as f32;
+      x += main_actor.x;
+      y += main_actor.y;
     }
 
+    //println!("render: {}: x,y = {}, {}", self.name, x, y);
+
     let mut transform: Matrix4<f32> = Matrix4::identity();
 
     // Apply orthographic projection matrix: left, right, bottom, top, near, far
@@ -518,17 +581,23 @@ impl<'a> Actor<'a> {
 
     for sub_actor in self.sub_actor_list.iter() {
       if sub_actor.focused == false {
-        sub_actor.render(shader_program, Some(&self));
+        sub_actor.render(shader_program, stretch,  Some(&self));
       }
     }
 
     // render the focused sub_actor at the end.
     if self.sub_actor_list.len() > 0 {
-      self.sub_actor_list[self.focused_sub_actor].render(shader_program, Some(&self));
+      self.sub_actor_list[self.focused_sub_actor].render(shader_program, stretch, Some(&self));
     }
   }
 
-  pub fn add_sub_actor(&mut self, actor: Actor<'a>) {
+  pub fn add_sub_actor(&mut self, mut actor: Actor<'a>) {
+    println!("{} : {}, {}", self.name, self.viewport_width, self.viewport_height);
+
+    // FIXME
+    // actor.init_gl(self.viewport_width, self.viewport_height);
+    actor.init_gl(1920, 1080);
+    //println!("stage::add_actor");
     self.sub_actor_list.push(actor);
   }
 }
